@@ -1,5 +1,5 @@
 //
-// "$Id: Fl_Group_Type.cxx 4553 2005-08-30 21:25:47Z matt $"
+// "$Id$"
 //
 // Fl_Group object code for the Fast Light Tool Kit (FLTK).
 //
@@ -7,31 +7,22 @@
 // the Fl_Tabs widget, with special stuff to select tab items and
 // insure that only one is visible.
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
 //     http://www.fltk.org/str.php
 //
 
-
 #include <FL/Fl.H>
 #include <FL/Fl_Group.H>
+#include <FL/Fl_Table.H>
 #include <FL/fl_message.H>
 #include "Fl_Widget_Type.h"
 #include "../src/flstring.h"
@@ -124,9 +115,12 @@ void Fl_Group_Type::write_code1() {
 }
 
 void Fl_Group_Type::write_code2() {
+  const char *var = name() ? name() : "o";
   write_extra_code();
-  write_c("%so->end();\n", indent());
-  if (resizable()) write_c("%sFl_Group::current()->resizable(o);\n", indent());
+  write_c("%s%s->end();\n", indent(), var);
+  if (resizable()) {
+    write_c("%sFl_Group::current()->resizable(%s);\n", indent(), var);
+  }
   write_block_close();
 }
 
@@ -143,6 +137,93 @@ Fl_Pack_Type Fl_Pack_type;	// the "factory"
 
 ////////////////////////////////////////////////////////////////
 
+static const int MAX_ROWS = 14;
+static const int MAX_COLS = 7;
+
+// this is a minimal table widget used as an example when adding tables in Fluid
+class Fluid_Table : public Fl_Table {
+  int data[MAX_ROWS][MAX_COLS];         // data array for cells
+  
+  // Draw the row/col headings
+  //    Make this a dark thin upbox with the text inside.
+  //
+  void DrawHeader(const char *s, int X, int Y, int W, int H) {
+    fl_push_clip(X,Y,W,H);
+    fl_draw_box(FL_THIN_UP_BOX, X,Y,W,H, row_header_color());
+    fl_color(FL_BLACK);
+    fl_draw(s, X,Y,W,H, FL_ALIGN_CENTER);
+    fl_pop_clip();
+  } 
+  // Draw the cell data
+  //    Dark gray text on white background with subtle border
+  //
+  void DrawData(const char *s, int X, int Y, int W, int H) {
+    fl_push_clip(X,Y,W,H);
+    // Draw cell bg
+    fl_color(FL_WHITE); fl_rectf(X,Y,W,H);
+    // Draw cell data
+    fl_color(FL_GRAY0); fl_draw(s, X,Y,W,H, FL_ALIGN_CENTER);
+    // Draw box border
+    fl_color(color()); fl_rect(X,Y,W,H);
+    fl_pop_clip();
+  } 
+  // Handle drawing table's cells
+  //     Fl_Table calls this function to draw each visible cell in the table.
+  //     It's up to us to use FLTK's drawing functions to draw the cells the way we want.
+  //
+  void draw_cell(TableContext context, int ROW=0, int COL=0, int X=0, int Y=0, int W=0, int H=0) {
+    static char s[40];
+    switch ( context ) {
+      case CONTEXT_STARTPAGE:                   // before page is drawn..
+        fl_font(FL_HELVETICA, 16);              // set the font for our drawing operations
+        return; 
+      case CONTEXT_COL_HEADER:                  // Draw column headers
+        sprintf(s,"%c",'A'+COL);                // "A", "B", "C", etc.
+        DrawHeader(s,X,Y,W,H);
+        return; 
+      case CONTEXT_ROW_HEADER:                  // Draw row headers
+        sprintf(s,"%03d:",ROW);                 // "001:", "002:", etc
+        DrawHeader(s,X,Y,W,H);
+        return; 
+      case CONTEXT_CELL:                        // Draw data in cells
+        sprintf(s,"%d",data[ROW][COL]);
+        DrawData(s,X,Y,W,H);
+        return;
+      default:
+        return;
+    }
+  }
+public:
+  Fluid_Table(int x, int y, int w, int h, const char *l=0L)
+  : Fl_Table(x, y, w, h, l) {
+    end();
+    for ( int r=0; r<MAX_ROWS; r++ )
+      for ( int c=0; c<MAX_COLS; c++ )
+        data[r][c] = 1000+(r*1000)+c;
+    // Rows
+    rows(MAX_ROWS);             // how many rows
+    row_header(1);              // enable row headers (along left)
+    row_height_all(20);         // default height of rows
+    row_resize(0);              // disable row resizing
+    // Cols
+    cols(MAX_COLS);             // how many columns
+    col_header(1);              // enable column headers (along top)
+    col_width_all(80);          // default width of columns
+    col_resize(1);              // enable column resizing
+  }
+};
+
+const char table_type_name[] = "Fl_Table";
+
+Fl_Table_Type Fl_Table_type;	// the "factory"
+
+Fl_Widget *Fl_Table_Type::widget(int X,int Y,int W,int H) {
+  Fluid_Table *table = new Fluid_Table(X, Y, W, H);
+  return table;
+}
+
+////////////////////////////////////////////////////////////////
+
 const char tabs_type_name[] = "Fl_Tabs";
 
 // Override group's resize behavior to do nothing to children:
@@ -161,10 +242,14 @@ Fl_Type* Fl_Tabs_Type::click_test(int x, int y) {
   Fl_Tabs *t = (Fl_Tabs*)o;
   Fl_Widget *a = t->which(x,y);
   if (!a) return 0; // didn't click on tab
+  // changing the visible tab has an impact on the generated
+  // source code, so mark this project as changed.
+  int changed = (a!=t->value());
   // okay, run the tabs ui until they let go of mouse:
   t->handle(FL_PUSH);
   Fl::pushed(t);
   while (Fl::pushed()==t) Fl::wait();
+  if (changed) set_modflag(1);
   return (Fl_Type*)(t->value()->user_data());
 }
 
@@ -194,6 +279,18 @@ void Fl_Tabs_Type::add_child(Fl_Type* c, Fl_Type* before) {
   Fl_Group_Type::add_child(c, before);
 }
 
+void Fl_Table_Type::add_child(Fl_Type* cc, Fl_Type* before) {
+  Fl_Widget_Type* c = (Fl_Widget_Type*)cc;
+  Fl_Widget* b = before ? ((Fl_Widget_Type*)before)->o : 0;
+  if (((Fl_Table*)o)->children()==1) { // the FLuid_Table has one extra child
+    fl_message("Inserting child widgets into an Fl_Table is not recommended.\n"
+               "Please refer to the documentation on Fl_Table.");
+  }
+  ((Fl_Table*)o)->insert(*(c->o), b);
+  o->redraw();
+}
+
+
 // This is called when o is deleted.  If it is in the tab group make
 // sure it is not visible:
 
@@ -210,6 +307,12 @@ void Fl_Tabs_Type::remove_child(Fl_Type* cc) {
   Fl_Group_Type::remove_child(c);
 }
 
+void Fl_Table_Type::remove_child(Fl_Type* cc) {
+  Fl_Widget_Type* c = (Fl_Widget_Type*)cc;
+  ((Fl_Table*)o)->remove(*(c->o));
+  o->redraw();
+}
+
 // move, don't change selected value:
 
 void Fl_Group_Type::move_child(Fl_Type* cc, Fl_Type* before) {
@@ -220,10 +323,18 @@ void Fl_Group_Type::move_child(Fl_Type* cc, Fl_Type* before) {
   o->redraw();
 }
 
+void Fl_Table_Type::move_child(Fl_Type* cc, Fl_Type* before) {
+  Fl_Widget_Type* c = (Fl_Widget_Type*)cc;
+  Fl_Widget* b = before ? ((Fl_Widget_Type*)before)->o : 0;
+  ((Fl_Table*)o)->remove(*(c->o));
+  ((Fl_Table*)o)->insert(*(c->o), b);
+  o->redraw();
+}
+
 ////////////////////////////////////////////////////////////////
 // live mode support
 
-Fl_Widget *Fl_Group_Type::enter_live_mode(int top) {
+Fl_Widget *Fl_Group_Type::enter_live_mode(int) {
   Fl_Group *grp = new Fl_Group(o->x(), o->y(), o->w(), o->h());
   live_widget = grp;
   if (live_widget) {
@@ -238,7 +349,7 @@ Fl_Widget *Fl_Group_Type::enter_live_mode(int top) {
   return live_widget;
 }
 
-Fl_Widget *Fl_Tabs_Type::enter_live_mode(int top) {
+Fl_Widget *Fl_Tabs_Type::enter_live_mode(int) {
   Fl_Tabs *grp = new Fl_Tabs(o->x(), o->y(), o->w(), o->h());
   live_widget = grp;
   if (live_widget) {
@@ -254,6 +365,16 @@ Fl_Widget *Fl_Tabs_Type::enter_live_mode(int top) {
   return live_widget;
 }
 
+Fl_Widget *Fl_Table_Type::enter_live_mode(int) {
+  Fl_Group *grp = new Fluid_Table(o->x(), o->y(), o->w(), o->h());
+  live_widget = grp;
+  if (live_widget) {
+    copy_properties();
+    grp->end();
+  }
+  return live_widget;
+}
+
 void Fl_Group_Type::leave_live_mode() {
 }
 
@@ -271,6 +392,22 @@ void Fl_Group_Type::copy_properties() {
 
 const char scroll_type_name[] = "Fl_Scroll";
 
+Fl_Widget *Fl_Scroll_Type::enter_live_mode(int) {
+  Fl_Group *grp = new Fl_Scroll(o->x(), o->y(), o->w(), o->h());
+  grp->show();
+  live_widget = grp;
+  if (live_widget) {
+    copy_properties();
+    Fl_Type *n;
+    for (n = next; n && n->level > level; n = n->next) {
+      if (n->level == level+1)
+        n->enter_live_mode();
+    }
+    grp->end();
+  }
+  return live_widget;
+}
+
 Fl_Menu_Item scroll_type_menu[] = {
   {"BOTH", 0, 0, 0/*(void*)Fl_Scroll::BOTH*/},
   {"HORIZONTAL", 0, 0, (void*)Fl_Scroll::HORIZONTAL},
@@ -303,5 +440,5 @@ void Fl_Tile_Type::copy_properties() {
 }
 
 //
-// End of "$Id: Fl_Group_Type.cxx 4553 2005-08-30 21:25:47Z matt $".
+// End of "$Id$".
 //
