@@ -620,6 +620,10 @@ double Fl::wait(double time_to_wait) {
     flush();
     if (idle && !in_idle) // 'idle' may have been set within flush()
       time_to_wait = 0.0;
+    else if (first_timeout && first_timeout->time < time_to_wait) {
+      // another timeout may have been queued within flush(), see STR #3188
+      time_to_wait = first_timeout->time >= 0.0 ? first_timeout->time : 0.0;
+    }
     return fl_wait(time_to_wait);
   }
 #endif
