@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Gl_Window.cxx 10214 2014-06-30 10:30:58Z ossman $"
+// "$Id$"
 //
 // OpenGL window code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2010 by Bill Spitzak and others.
+// Copyright 1998-2015 by Bill Spitzak and others.
 //
 // This library is free software. Distribution and use rights are outlined in
 // the file "COPYING" which should have been included with this file.  If this
@@ -20,16 +20,14 @@
 #if HAVE_GL
 
 extern int fl_gl_load_plugin;
-#ifdef __APPLE__
-extern void gl_texture_reset();
-#endif
 
 #include <FL/Fl.H>
 #include <FL/x.H>
+#include "Fl_Gl_Choice.H"
 #ifdef __APPLE__
 #include <FL/gl.h>
+#include <OpenGL/OpenGL.h>
 #endif
-#include "Fl_Gl_Choice.H"
 #include <FL/Fl_Gl_Window.H>
 #include <stdlib.h>
 #include <FL/fl_utf8.h>
@@ -71,7 +69,6 @@ void Fl_Gl_Window::show() {
   if (!shown()) {
     if (!g) {
       g = Fl_Gl_Choice::find(mode_,alist);
-
       if (!g && (mode_ & FL_DOUBLE) == FL_SINGLE) {
         g = Fl_Gl_Choice::find(mode_ | FL_DOUBLE,alist);
 	if (g) mode_ |= FL_FAKE_SINGLE;
@@ -97,6 +94,15 @@ void Fl_Gl_Window::show() {
 #endif /* __APPLE__ */
 }
 
+#if defined(__APPLE__)
+
+float Fl_Gl_Window::pixels_per_unit()
+{
+  return (fl_mac_os_version >= 100700 && Fl::use_high_res_GL() && Fl_X::i(this) && Fl_X::i(this)->mapped_to_retina()) ? 2 : 1;
+}
+
+#endif // __APPLE__
+
 /**
   The invalidate() method turns off valid() and is
   equivalent to calling value(0).
@@ -112,13 +118,25 @@ void Fl_Gl_Window::invalidate() {
 #endif
 }
 
-/**
-  See const int Fl_Gl_Window::mode() const 
-*/
 int Fl_Gl_Window::mode(int m, const int *a) {
   if (m == mode_ && a == alist) return 0;
 #ifndef __APPLE__
   int oldmode = mode_;
+#endif
+#if defined(__APPLE__) || defined(USE_X11)
+  if (a) { // when the mode is set using the a array of system-dependent values, and if asking for double buffer,
+           // the FL_DOUBLE flag must be set in the mode_ member variable
+    const int *aa = a;
+    while (*aa) {
+      if (*(aa++) ==
+#  if defined(__APPLE__)
+          kCGLPFADoubleBuffer
+#  else
+          GLX_DOUBLEBUFFER
+#  endif
+          ) { m |= FL_DOUBLE; break; }
+    }
+  }
 #endif // !__APPLE__
 #if !defined(WIN32) && !defined(__APPLE__)
   Fl_Gl_Choice* oldg = g;
@@ -140,7 +158,6 @@ int Fl_Gl_Window::mode(int m, const int *a) {
       show();
     }
 #elif defined(__APPLE_QUARTZ__)
-    // warning: the Quartz version should probably use Core GL (CGL) instead of AGL
     redraw();
 #else
 #  error unsupported platform
@@ -163,41 +180,22 @@ int Fl_Gl_Window::mode(int m, const int *a) {
 void Fl_Gl_Window::make_current() {
 //  puts("Fl_Gl_Window::make_current()");
 //  printf("make_current: context_=%p\n", context_);
+#if defined(__APPLE__)
+  // detect if the window was moved between low and high resolution displays
+  if (Fl_X::i(this)->changed_resolution()){
+    Fl_X::i(this)->changed_resolution(false);
+    invalidate();
+    Fl_X::GLcontext_update(context_);
+  }
+#endif
   if (!context_) {
     mode_ &= ~NON_LOCAL_CONTEXT;
     context_ = fl_create_gl_context(this, g);
     valid(0);
     context_valid(0);
-#ifdef __APPLE__
-    // resets the pile of string textures used to draw strings
-    gl_texture_reset();
-#endif
   }
   fl_set_gl_context(this, context_);
 
-#ifdef __APPLE__
-  // Set the buffer rectangle here, since in resize() we won't have the
-  // correct parent window size to work with...
-  GLint xywh[4];
-
-  if (window()) {
-    int xoff,yoff;
-    const Fl_Window *win = top_window_offset(xoff, yoff);	// STR #2944 [2]
-    xywh[0] = xoff;
-    xywh[1] = win->h() - yoff - h();
-  } else {
-    xywh[0] = 0;
-    xywh[1] = 0;
-  }
-
-  xywh[2] = w();
-  xywh[3] = h();
-
-  aglSetInteger(context_, AGL_BUFFER_RECT, xywh);
-  aglEnable(context_, AGL_BUFFER_RECT);
-//  printf("make_current: xywh=[%d %d %d %d]\n", xywh[0], xywh[1], xywh[2], xywh[3]);
-#endif // __APPLE__
-
 #if defined(WIN32) && USE_COLORMAP
   if (fl_palette) {
     fl_GetDC(fl_xid(this));
@@ -227,8 +225,8 @@ void Fl_Gl_Window::ortho() {
   GLint v[2];
   glGetIntegerv(GL_MAX_VIEWPORT_DIMS, v);
   glLoadIdentity();
-  glViewport(w()-v[0], h()-v[1], v[0], v[1]);
-  glOrtho(w()-v[0], w(), h()-v[1], h(), -1, 1);
+  glViewport(pixel_w()-v[0], pixel_h()-v[1], v[0], v[1]);
+  glOrtho(pixel_w()-v[0], pixel_w(), pixel_h()-v[1], pixel_h(), -1, 1);
 #endif
 }
 
@@ -252,7 +250,7 @@ void Fl_Gl_Window::swap_buffers() {
     // STR# 2944 [1]
     //    Save matrixmode/proj/modelview/rasterpos before doing overlay.
     //
-    int wo=w(), ho=h();
+    int wo=pixel_w(), ho=pixel_h();
     GLint matrixmode;
     GLfloat pos[4];
     glGetIntegerv(GL_MATRIX_MODE, &matrixmode);
@@ -278,8 +276,10 @@ void Fl_Gl_Window::swap_buffers() {
     glMatrixMode(matrixmode);
     glRasterPos3f(pos[0], pos[1], pos[2]);              // restore original glRasterPos
   }
-  else
+  /* // nothing to do here under Cocoa because [NSOpenGLContext -flushBuffer] done later replaces it
+   else
     aglSwapBuffers((AGLContext)context_);
+   */
 #else
 # error unsupported platform
 #endif
@@ -297,18 +297,13 @@ void Fl_Gl_Window::flush() {
 #if HAVE_GL_OVERLAY && defined(WIN32)
   uchar save_valid_f = valid_f_;
 #endif
-
-#if defined(__APPLE_QUARTZ__)
-  // warning: the Quartz version should probably use Core GL (CGL) instead of AGL
-  //: clear previous clipping in this shared port
-#if ! __LP64__
-/*GrafPtr port = GetWindowPort( Fl_X::i(this)->window_ref() );
-  Rect rect; SetRect( &rect, 0, 0, 0x7fff, 0x7fff );
-  GrafPtr old; GetPort( &old );
-  SetPort( port );
-  ClipRect( &rect );
-  SetPort( old );*/
-#endif
+  
+#ifdef __APPLE__
+  Fl_X *i = Fl_X::i(this);
+  if (i->wait_for_expose) {
+    Fl_X::GLcontext_update((GLContext)context());
+    i->wait_for_expose = 0;
+  }
 #endif
 
 #if HAVE_GL_OVERLAY && defined(WIN32)
@@ -391,22 +386,25 @@ void Fl_Gl_Window::flush() {
 	  glReadBuffer(GL_BACK);
 	  glDrawBuffer(GL_FRONT);
 	  glLoadIdentity();
-	  glViewport(0, 0, w(), h());
-	  glOrtho(0, w(), 0, h(), -1, 1);
+	  glViewport(0, 0, pixel_w(), pixel_h());
+	  glOrtho(0, pixel_w(), 0, pixel_h(), -1, 1);
 	  glRasterPos2i(0,0);
 	  ortho_window = this;
 	}
-	glCopyPixels(0,0,w(),h(),GL_COLOR);
+	glCopyPixels(0,0,pixel_w(),pixel_h(),GL_COLOR);
 	make_current(); // set current context back to draw overlay
 	damage1_ = 0;
 
       } else {
-	damage1_ = damage();
-	clear_damage(0xff); draw();
-	swap_buffers();
+        damage1_ = damage();
+        clear_damage(0xff); draw();
+        swap_buffers();
       }
 
     }
+#ifdef __APPLE__
+    Fl_X::GLcontext_flushbuffer(context_);
+#endif
 
     if (overlay==this && SWAP_TYPE != SWAP) { // fake overlay in front buffer
       glDrawBuffer(GL_FRONT);
@@ -431,12 +429,16 @@ void Fl_Gl_Window::resize(int X,int Y,int W,int H) {
 //  printf("Fl_Gl_Window::resize(X=%d, Y=%d, W=%d, H=%d)\n", X, Y, W, H);
 //  printf("current: x()=%d, y()=%d, w()=%d, h()=%d\n", x(), y(), w(), h());
 
-  if (W != w() || H != h()) valid(0);
-
+  int is_a_resize = (W != Fl_Widget::w() || H != Fl_Widget::h());
+  if (is_a_resize) valid(0);
+  
 #ifdef __APPLE__
-  if (X != x() || Y != y() || W != w() || H != h()) aglUpdateContext(context_);
-#elif !defined(WIN32)
-  if ((W != w() || H != h()) && !resizable() && overlay && overlay != this) {
+  Fl_X *flx = Fl_X::i(this);
+  if (flx && flx->in_windowDidResize()) Fl_X::GLcontext_update(context_);
+#endif
+
+#if ! ( defined(__APPLE__) || defined(WIN32) )
+  if (is_a_resize && !resizable() && overlay && overlay != this) {
     ((Fl_Gl_Window*)overlay)->resize(0,0,W,H);
   }
 #endif
@@ -445,8 +447,8 @@ void Fl_Gl_Window::resize(int X,int Y,int W,int H) {
 }
 
 /**
-  Returns or sets a pointer to the GLContext that this window is
-  using. This is a system-dependent structure, but it is portable to copy
+  Sets a pointer to the GLContext that this window is using.
+  This is a system-dependent structure, but it is portable to copy
   the context from one window to another. You can also set it to NULL,
   which will force FLTK to recreate the context the next time make_current()
   is called, this is useful for getting around bugs in OpenGL implementations.
@@ -548,18 +550,6 @@ void Fl_Gl_Window::draw() {
  */
 int Fl_Gl_Window::handle(int event) 
 {
-#ifdef __APPLE_QUARTZ__
-  if (event==FL_HIDE) {
-    // if we are not hidden, just the parent was hidden, so we must throw away the context
-    if (!visible_r())
-      context(0); // remove context without setting the hidden flags
-  }
-  if (event==FL_SHOW) {
-    // if we are not hidden, just the parent was shown, so we must create a new context
-    if (visible_r())
-      show(); //
-  }
-#endif
   return Fl_Window::handle(event);
 }
 
@@ -569,5 +559,5 @@ int Fl_Gl_Window::gl_plugin_linkage() {
 }
 
 //
-// End of "$Id: Fl_Gl_Window.cxx 10214 2014-06-30 10:30:58Z ossman $".
+// End of "$Id$".
 //
