@@ -1,24 +1,15 @@
 //
-// "$Id: Fl_Menu.cxx 6478 2008-10-19 21:28:06Z matt $"
+// "$Id$"
 //
 // Menu code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2006 by Bill Spitzak and others.
+// Copyright 1998-2015 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -38,16 +29,38 @@
 #include <stdio.h>
 #include "flstring.h"
 
-#ifdef __APPLE__
-#  include <Carbon/Carbon.h>
-#endif
+/** Size of the menu starting from this menu item.
 
+  This method counts all menu items starting with \p this menu item,
+  including all menu items in the same (sub)menu level, all nested
+  submenus, \b and the terminating empty (0) menu item.
+
+  It does \b not count menu items referred to by FL_SUBMENU_POINTER
+  menu items (except the single menu item with FL_SUBMENU_POINTER).
+
+  All menu items counted are consecutive in memory (one array).
+
+  Example:
+
+  \code
+    schemechoice = new Fl_Choice(X+125,Y,140,25,"FLTK Scheme");
+    schemechoice->add("none");
+    schemechoice->add("plastic");
+    schemechoice->add("gtk+");
+    schemechoice->add("gleam");
+    printf("schemechoice->menu()->size() = %d\n", schemechoice->menu()->size());
+  \endcode
+
+  Output:
+
+  schemechoice->menu()->%size() = 5
+*/
 int Fl_Menu_Item::size() const {
   const Fl_Menu_Item* m = this;
   int nest = 0;
   for (;;) {
     if (!m->text) {
-      if (!nest) return (m-this+1);
+      if (!nest) return (int) (m-this+1);
       nest--;
     } else if (m->flags & FL_SUBMENU) {
       nest++;
@@ -56,12 +69,11 @@ int Fl_Menu_Item::size() const {
   }
 }
 
-const Fl_Menu_Item* Fl_Menu_Item::next(int n) const {
-  if (n < 0) return 0; // this is so selected==-1 returns NULL
-  const Fl_Menu_Item* m = this;
+// Advance a pointer to next visible or invisible item of a menu array, 
+// skipping the contents of submenus.
+static const Fl_Menu_Item* next_visible_or_not(const Fl_Menu_Item* m) {
   int nest = 0;
-  if (!m->visible()) n++;
-  while (n>0) {
+  do {
     if (!m->text) {
       if (!nest) return m;
       nest--;
@@ -69,13 +81,29 @@ const Fl_Menu_Item* Fl_Menu_Item::next(int n) const {
       nest++;
     }
     m++;
-    if (!nest && m->visible()) n--;
+  }
+  while (nest);
+  return m;
+}
+
+/**
+  Advance a pointer by n items through a menu array, skipping
+  the contents of submenus and invisible items.  There are two calls so
+  that you can advance through const and non-const data.
+*/
+const Fl_Menu_Item* Fl_Menu_Item::next(int n) const {
+  if (n < 0) return 0; // this is so selected==-1 returns NULL
+  const Fl_Menu_Item* m = this;
+  if (!m->visible()) n++;
+  while (n) {
+    m = next_visible_or_not(m);
+    if (m->visible() || !m->text) n--;
   }
   return m;
 }
 
 // appearance of current menus are pulled from this parent widget:
-static const Fl_Menu_* button;
+static const Fl_Menu_* button=0;
 
 ////////////////////////////////////////////////////////////////
 
@@ -94,13 +122,14 @@ class menuwindow : public Fl_Menu_Window {
 public:
   menutitle* title;
   int handle(int);
-#ifdef __APPLE__
+#if defined (__APPLE__) || defined (USE_X11)
   int early_hide_handle(int);
 #endif
   int itemheight;	// zero == menubar
   int numitems;
   int selected;
   int drawn_selected;	// last redraw has this selected
+  int shortcutWidth;
   const Fl_Menu_Item* menu;
   menuwindow(const Fl_Menu_Item* m, int X, int Y, int W, int H,
 	     const Fl_Menu_Item* picked, const Fl_Menu_Item* title,
@@ -118,24 +147,28 @@ public:
 
 extern char fl_draw_shortcut;
 
-// width of label, including effect of & characters:
+/** 
+  Measures width of label, including effect of & characters. 
+  Optionally, can get height if hp is not NULL. 
+*/
 int Fl_Menu_Item::measure(int* hp, const Fl_Menu_* m) const {
   Fl_Label l;
   l.value   = text;
   l.image   = 0;
   l.deimage = 0;
   l.type    = labeltype_;
-  l.font    = labelsize_ || labelfont_ ? labelfont_ : uchar(m ? m->textfont() : FL_HELVETICA);
-  l.size    = labelsize_ ? labelsize_ : m ? m->textsize() : (uchar)FL_NORMAL_SIZE;
+  l.font    = labelsize_ || labelfont_ ? labelfont_ : (m ? m->textfont() : FL_HELVETICA);
+  l.size    = labelsize_ ? labelsize_ : m ? m->textsize() : FL_NORMAL_SIZE;
   l.color   = FL_FOREGROUND_COLOR; // this makes no difference?
   fl_draw_shortcut = 1;
   int w = 0; int h = 0;
   l.measure(w, hp ? *hp : h);
   fl_draw_shortcut = 0;
-  if (flags & (FL_MENU_TOGGLE|FL_MENU_RADIO)) w += 14;
+  if (flags & (FL_MENU_TOGGLE|FL_MENU_RADIO)) w += FL_NORMAL_SIZE;
   return w;
 }
 
+/** Draws the menu item in bounding box x,y,w,h, optionally selects the item. */
 void Fl_Menu_Item::draw(int x, int y, int w, int h, const Fl_Menu_* m,
 			int selected) const {
   Fl_Label l;
@@ -143,15 +176,15 @@ void Fl_Menu_Item::draw(int x, int y, int w, int h, const Fl_Menu_* m,
   l.image   = 0;
   l.deimage = 0;
   l.type    = labeltype_;
-  l.font    = labelsize_ || labelfont_ ? labelfont_ : uchar(m ? m->textfont() : FL_HELVETICA);
-  l.size    = labelsize_ ? labelsize_ : m ? m->textsize() : (uchar)FL_NORMAL_SIZE;
+  l.font    = labelsize_ || labelfont_ ? labelfont_ : (m ? m->textfont() : FL_HELVETICA);
+  l.size    = labelsize_ ? labelsize_ : m ? m->textsize() : FL_NORMAL_SIZE;
   l.color   = labelcolor_ ? labelcolor_ : m ? m->textcolor() : int(FL_FOREGROUND_COLOR);
   if (!active()) l.color = fl_inactive((Fl_Color)l.color);
   Fl_Color color = m ? m->color() : FL_GRAY;
   if (selected) {
     Fl_Color r = m ? m->selection_color() : FL_SELECTION_COLOR;
     Fl_Boxtype b = m && m->down_box() ? m->down_box() : FL_FLAT_BOX;
-    if (fl_contrast(r,color)!=r) { // back compatability boxtypes
+    if (fl_contrast(r,color)!=r) { // back compatibility boxtypes
       if (selected == 2) { // menu title
 	r = color;
 	b = m ? m->box() : FL_UP_BOX;
@@ -180,18 +213,12 @@ void Fl_Menu_Item::draw(int x, int y, int w, int h, const Fl_Menu_* m,
       if (value()) {
 	int tW = (W - Fl::box_dw(FL_ROUND_DOWN_BOX)) / 2 + 1;
 	if ((W - tW) & 1) tW++;	// Make sure difference is even to center
-	int td = Fl::box_dx(FL_ROUND_DOWN_BOX) + 1;
-        if (Fl::scheme()) {
-	  // Offset the radio circle...
-	  td ++;
-
-	  if (!strcmp(Fl::scheme(), "gtk+")) {
-	    fl_color(FL_SELECTION_COLOR);
-	    tW --;
-	    fl_pie(x + td + 1, y + d + td - 1, tW + 3, tW + 3, 0.0, 360.0);
-	    fl_arc(x + td + 1, y + d + td - 1, tW + 3, tW + 3, 0.0, 360.0);
-	    fl_color(fl_color_average(FL_WHITE, FL_SELECTION_COLOR, 0.2f));
-	  } else fl_color(labelcolor_);
+	int td = (W - tW) / 2;
+        if (Fl::is_scheme("gtk+")) {
+	  fl_color(FL_SELECTION_COLOR);
+	  tW --;
+	  fl_pie(x + td + 1, y + d + td - 1, tW + 3, tW + 3, 0.0, 360.0);
+	  fl_color(fl_color_average(FL_WHITE, FL_SELECTION_COLOR, 0.2f));
 	} else fl_color(labelcolor_);
 
 	switch (tW) {
@@ -220,7 +247,7 @@ void Fl_Menu_Item::draw(int x, int y, int w, int h, const Fl_Menu_* m,
 	    break;
 	}
 
-	if (Fl::scheme() && !strcmp(Fl::scheme(), "gtk+")) {
+	if (Fl::is_scheme("gtk+")) {
 	  fl_color(fl_color_average(FL_WHITE, FL_SELECTION_COLOR, 0.5));
 	  fl_arc(x + td + 2, y + d + td, tW + 1, tW + 1, 60.0, 180.0);
 	}
@@ -228,7 +255,7 @@ void Fl_Menu_Item::draw(int x, int y, int w, int h, const Fl_Menu_* m,
     } else {
       fl_draw_box(FL_DOWN_BOX, x+2, y+d, W, W, FL_BACKGROUND2_COLOR);
       if (value()) {
-	if (Fl::scheme() && !strcmp(Fl::scheme(), "gtk+")) {
+	if (Fl::is_scheme("gtk+")) {
 	  fl_color(FL_SELECTION_COLOR);
 	} else {
 	  fl_color(labelcolor_);
@@ -258,6 +285,7 @@ menutitle::menutitle(int X, int Y, int W, int H, const Fl_Menu_Item* L) :
   end();
   set_modal();
   clear_border();
+  set_menu_window();
   menu = L;
   if (L->labelcolor_ || Fl::scheme() || L->labeltype_ > FL_NO_LABEL) clear_overlay();
 }
@@ -270,12 +298,13 @@ menuwindow::menuwindow(const Fl_Menu_Item* m, int X, int Y, int Wp, int Hp,
   int scr_x, scr_y, scr_w, scr_h;
   int tx = X, ty = Y;
 
-  Fl::screen_xywh(scr_x, scr_y, scr_w, scr_h);
+  Fl::screen_work_area(scr_x, scr_y, scr_w, scr_h);
   if (!right_edge || right_edge > scr_x+scr_w) right_edge = scr_x+scr_w;
 
   end();
   set_modal();
   clear_border();
+  set_menu_window();
   menu = m;
   if (m) m = m->first(); // find the first item that needs to be rendered
   drawn_selected = -1;
@@ -287,11 +316,12 @@ menuwindow::menuwindow(const Fl_Menu_Item* m, int X, int Y, int Wp, int Hp,
   }
   color(button && !Fl::scheme() ? button->color() : FL_GRAY);
   selected = -1;
-  {int j = 0;
-  if (m) for (const Fl_Menu_Item* m1=m; ; m1 = m1->next(), j++) {
-    if (picked) {
-      if (m1 == picked) {selected = j; picked = 0;}
-      else if (m1 > picked) {selected = j-1; picked = 0; Wp = Hp = 0;}
+  {
+    int j = 0;
+    if (m) for (const Fl_Menu_Item* m1=m; ; m1 = m1->next(), j++) {
+      if (picked) {
+        if (m1 == picked) {selected = j; picked = 0;}
+        else if (m1 > picked) {selected = j-1; picked = 0; Wp = Hp = 0;}
     }
     if (!m1->text) break;
   }
@@ -306,48 +336,72 @@ menuwindow::menuwindow(const Fl_Menu_Item* m, int X, int Y, int Wp, int Hp,
   itemheight = 1;
 
   int hotKeysw = 0;
+  int hotModsw = 0;
   int Wtitle = 0;
   int Htitle = 0;
   if (t) Wtitle = t->measure(&Htitle, button) + 12;
   int W = 0;
   if (m) for (; m->text; m = m->next()) {
-    int hh; int w1 = m->measure(&hh, button);
+    int hh; 
+    int w1 = m->measure(&hh, button);
     if (hh+LEADING>itemheight) itemheight = hh+LEADING;
-    if (m->flags&(FL_SUBMENU|FL_SUBMENU_POINTER)) w1 += 14;
+    if (m->flags&(FL_SUBMENU|FL_SUBMENU_POINTER)) 
+      w1 += FL_NORMAL_SIZE;
     if (w1 > W) W = w1;
+    // calculate the maximum width of all shortcuts
     if (m->shortcut_) {
-      w1 = int(fl_width(fl_shortcut_label(m->shortcut_))) + 8;
-      if (w1 > hotKeysw) hotKeysw = w1;
+      // s is a pointer to the UTF-8 string for the entire shortcut
+      // k points only to the key part (minus the modifier keys)
+      const char *k, *s = fl_shortcut_label(m->shortcut_, &k);
+      if (fl_utf_nb_char((const unsigned char*)k, (int) strlen(k))<=4) {
+        // a regular shortcut has a right-justified modifier followed by a left-justified key
+        w1 = int(fl_width(s, (int) (k-s)));
+        if (w1 > hotModsw) hotModsw = w1;
+        w1 = int(fl_width(k))+4;
+        if (w1 > hotKeysw) hotKeysw = w1;
+      } else {
+        // a shortcut with a long modifier is right-justified to the menu
+        w1 = int(fl_width(s))+4;
+        if (w1 > (hotModsw+hotKeysw)) {
+          hotModsw = w1-hotKeysw;
+        }
+      }
     }
     if (m->labelcolor_ || Fl::scheme() || m->labeltype_ > FL_NO_LABEL) clear_overlay();
   }
+  shortcutWidth = hotKeysw;
   if (selected >= 0 && !Wp) X -= W/2;
   int BW = Fl::box_dx(box());
-  W += hotKeysw+2*BW+7;
+  W += hotKeysw+hotModsw+2*BW+7;
   if (Wp > W) W = Wp;
   if (Wtitle > W) W = Wtitle;
 
-  if (X < scr_x) X = scr_x; if (X > scr_x+scr_w-W) X = right_edge-W; // X= scr_x+scr_w-W;
+  if (X < scr_x) X = scr_x; 
+  // this change improves popup submenu positioning at right screen edge, 
+  // but it makes right_edge argument useless
+  //if (X > scr_x+scr_w-W) X = right_edge-W;
+  if (X > scr_x+scr_w-W) X = scr_x+scr_w-W;
   x(X); w(W);
   h((numitems ? itemheight*numitems-LEADING : 0)+2*BW+3);
-  if (selected >= 0)
+  if (selected >= 0) {
     Y = Y+(Hp-itemheight)/2-selected*itemheight-BW;
-  else {
+  } else {
     Y = Y+Hp;
     // if the menu hits the bottom of the screen, we try to draw
     // it above the menubar instead. We will not adjust any menu
     // that has a selected item.
     if (Y+h()>scr_y+scr_h && Y-h()>=scr_y) {
-      if (Hp>1) 
+      if (Hp>1) {
         // if we know the height of the Fl_Menu_, use it
         Y = Y-Hp-h();
-      else if (t)
+      } else if (t) {
         // assume that the menubar item height relates to the first
         // menuitem as well
         Y = Y-itemheight-h()-Fl::box_dh(box());
-      else
+      } else {
         // draw the menu to the right
         Y = Y-h()+itemheight+Fl::box_dy(box());
+      }
     }
   }
   if (m) y(Y); else {y(Y-2); w(1); h(1);}
@@ -362,8 +416,9 @@ menuwindow::menuwindow(const Fl_Menu_Item* m, int X, int Y, int Wp, int Hp,
       int ht = Htitle+2*BW+3;
       title = new menutitle(X, Y-ht-dy, Wtitle, ht, t);
     }
-  } else
+  } else {
     title = 0;
+  }
 }
 
 menuwindow::~menuwindow() {
@@ -379,10 +434,11 @@ void menuwindow::position(int X, int Y) {
 
 // scroll so item i is visible on screen
 void menuwindow::autoscroll(int n) {
-  int scr_x, scr_y, scr_w, scr_h;
+  int scr_y, scr_h;
   int Y = y()+Fl::box_dx(box())+2+n*itemheight;
 
-  Fl::screen_xywh(scr_x, scr_y, scr_w, scr_h);
+  int xx, ww;
+  Fl::screen_work_area(xx, scr_y, ww, scr_h);
   if (Y <= scr_y) Y = scr_y-Y+10;
   else {
     Y = Y+itemheight-scr_h-scr_y;
@@ -413,7 +469,7 @@ void menuwindow::drawentry(const Fl_Menu_Item* m, int n, int eraseit) {
 
   m->draw(xx, yy, ww, hh, button, n==selected);
 
-  // the shortcuts and arrows assumme fl_color() was left set by draw():
+  // the shortcuts and arrows assume fl_color() was left set by draw():
   if (m->submenu()) {
     int sz = (hh-7)&-2;
     int y1 = yy+(hh-sz)/2;
@@ -424,7 +480,18 @@ void menuwindow::drawentry(const Fl_Menu_Item* m, int n, int eraseit) {
                     button ? button->textfont() : FL_HELVETICA;
     fl_font(f, m->labelsize_ ? m->labelsize_ :
                    button ? button->textsize() : FL_NORMAL_SIZE);
-    fl_draw(fl_shortcut_label(m->shortcut_), xx, yy, ww-3, hh, FL_ALIGN_RIGHT);
+    const char *k, *s = fl_shortcut_label(m->shortcut_, &k);
+    if (fl_utf_nb_char((const unsigned char*)k, (int) strlen(k))<=4) {
+      // right-align the modifiers and left-align the key
+      char *buf = (char*)malloc(k-s+1);
+      memcpy(buf, s, k-s); buf[k-s] = 0;
+      fl_draw(buf, xx, yy, ww-shortcutWidth, hh, FL_ALIGN_RIGHT);
+      fl_draw(  k, xx+ww-shortcutWidth, yy, shortcutWidth, hh, FL_ALIGN_LEFT);
+      free(buf);
+    } else {
+      // right-align to the menu
+      fl_draw(s, xx, yy, ww-4, hh, FL_ALIGN_RIGHT);
+    }
   }
 
   if (m->flags & FL_MENU_DIVIDER) {
@@ -496,6 +563,10 @@ int menuwindow::is_inside(int mx, int my) {
        my < y_root() || my >= y_root() + h()) {
     return 0;
   }
+  if (itemheight == 0 && find_selected(mx, my) == -1) {
+    // in the menubar but out from any menu header
+    return 0;
+    }
   return 1;
 }
 
@@ -532,7 +603,7 @@ struct menustate {
   menuwindow* fakemenu; // kludge for buttons in menubar
   int is_inside(int mx, int my);
 };
-static menustate* p;
+static menustate* p=0;
 
 // return 1 if the coordinates are inside any of the menuwindows
 int menustate::is_inside(int mx, int my) {
@@ -559,7 +630,7 @@ static void setitem(int m, int n) {
 
 static int forward(int menu) { // go to next item in menu menu if possible
   menustate &pp = *p;
-  // Fl_Menu_Button can geberate menu=-1. This line fixes it and selectes the first item.
+  // Fl_Menu_Button can generate menu=-1. This line fixes it and selectes the first item.
   if (menu==-1) 
     menu = 0;
   menuwindow &m = *(pp.p[menu]);
@@ -584,7 +655,7 @@ static int backward(int menu) { // previous item in menu menu if possible
 }
 
 int menuwindow::handle(int e) {
-#ifdef __APPLE__
+#if defined (__APPLE__) || defined (USE_X11)
   // This off-route takes care of the "detached menu" bug on OS X.
   // Apple event handler requires that we hide all menu windows right
   // now, so that Carbon can continue undisturbed with handling window
@@ -618,7 +689,6 @@ int menuwindow::early_hide_handle(int e) {
   case FL_KEYBOARD:
     switch (Fl::event_key()) {
     case FL_BackSpace:
-    case 0xFE20: // backtab
     BACKTAB:
       if (!backward(pp.menu_number)) {pp.item_number = -1;backward(pp.menu_number);}
       return 1;
@@ -644,7 +714,7 @@ int menuwindow::early_hide_handle(int e) {
       }
       return 1;
     case FL_Right:
-      if (pp.menubar && (pp.menu_number<=0 || pp.menu_number==1 && pp.nummenus==2))
+      if (pp.menubar && (pp.menu_number<=0 || (pp.menu_number==1 && pp.nummenus==2)))
 	forward(0);
       else if (pp.menu_number < pp.nummenus-1) forward(pp.menu_number+1);
       return 1;
@@ -664,70 +734,78 @@ int menuwindow::early_hide_handle(int e) {
       return 1;
     }
     break;
-  case FL_SHORTCUT: {
-    for (int mymenu = pp.nummenus; mymenu--;) {
-      menuwindow &mw = *(pp.p[mymenu]);
-      int item; const Fl_Menu_Item* m = mw.menu->find_shortcut(&item);
-      if (m) {
-	setitem(m, mymenu, item);
-	if (!m->submenu()) pp.state = DONE_STATE;
+  case FL_SHORTCUT: 
+    {
+      for (int mymenu = pp.nummenus; mymenu--;) {
+	menuwindow &mw = *(pp.p[mymenu]);
+	int item; const Fl_Menu_Item* m = mw.menu->find_shortcut(&item);
+	if (m) {
+	  setitem(m, mymenu, item);
+	  if (!m->submenu()) pp.state = DONE_STATE;
+	  return 1;
+	}
+      }
+    }
+    break;
+    case FL_MOVE:
+#if ! (defined(WIN32) || defined(__APPLE__))
+      if (pp.state == DONE_STATE) {
+	return 1; // Fix for STR #2619
+      }
+      /* FALLTHROUGH */
+#endif
+  case FL_ENTER:
+  case FL_PUSH:
+  case FL_DRAG:
+    {
+      int mx = Fl::event_x_root();
+      int my = Fl::event_y_root();
+      int item=0; int mymenu = pp.nummenus-1;
+      // Clicking or dragging outside menu cancels it...
+      if ((!pp.menubar || mymenu) && !pp.is_inside(mx, my)) {
+	setitem(0, -1, 0);
+	if (e==FL_PUSH)
+	  pp.state = DONE_STATE;
 	return 1;
       }
-    }} break;
-  case FL_ENTER:
-  case FL_MOVE:
-  case FL_PUSH:
-  case FL_DRAG: {
-#ifdef __QNX__
-    // STR 704: workaround QNX X11 bug - in QNX a FL_MOVE event is sent
-    // right after FL_RELEASE...
-    if (pp.state == DONE_STATE) return 1;
-#endif // __QNX__
-    int mx = Fl::event_x_root();
-    int my = Fl::event_y_root();
-    int item=0; int mymenu = pp.nummenus-1;
-    // Clicking or dragging outside menu cancels it...
-    if ((!pp.menubar || mymenu) && !pp.is_inside(mx, my)) {
-      setitem(0, -1, 0);
-      if (e==FL_PUSH)
-        pp.state = DONE_STATE;
-      return 1;
-    }
-    for (mymenu = pp.nummenus-1; ; mymenu--) {
-      item = pp.p[mymenu]->find_selected(mx, my);
-      if (item >= 0) 
-        break;
-      if (mymenu <= 0) {
-        // buttons in menubars must be deselected if we move outside of them!
-        if (pp.menu_number==-1 && e==FL_PUSH) {
-          pp.state = DONE_STATE;
-          return 1;
-        }
-        if (pp.current_item && pp.menu_number==0 && !pp.current_item->submenu()) {
-          if (e==FL_PUSH)
-            pp.state = DONE_STATE;
-          setitem(0, -1, 0);
-          return 1;
-        }
-        // all others can stay selected
-        return 0;
+      for (mymenu = pp.nummenus-1; ; mymenu--) {
+	item = pp.p[mymenu]->find_selected(mx, my);
+	if (item >= 0) 
+	  break;
+	if (mymenu <= 0) {
+	  // buttons in menubars must be deselected if we move outside of them!
+	  if (pp.menu_number==-1 && e==FL_PUSH) {
+	    pp.state = DONE_STATE;
+	    return 1;
+	  }
+	  if (pp.current_item && pp.menu_number==0 && !pp.current_item->submenu()) {
+	    if (e==FL_PUSH)
+	      pp.state = DONE_STATE;
+	    setitem(0, -1, 0);
+	    return 1;
+	  }
+	  // all others can stay selected
+	  return 0;
+	}
+      }
+      if (my == 0 && item > 0) setitem(mymenu, item - 1);
+      else setitem(mymenu, item);
+      if (e == FL_PUSH) {
+	if (pp.current_item && pp.current_item->submenu() // this is a menu title
+	    && item != pp.p[mymenu]->selected // and it is not already on
+	    && !pp.current_item->callback_) // and it does not have a callback
+	  pp.state = MENU_PUSH_STATE;
+	else
+	  pp.state = PUSH_STATE;
       }
     }
-    if (my == 0 && item > 0) setitem(mymenu, item - 1);
-    else setitem(mymenu, item);
-    if (e == FL_PUSH) {
-      if (pp.current_item && pp.current_item->submenu() // this is a menu title
-	  && item != pp.p[mymenu]->selected // and it is not already on
-	  && !pp.current_item->callback_) // and it does not have a callback
-	pp.state = MENU_PUSH_STATE;
-      else
-	pp.state = PUSH_STATE;
-    }} return 1;
+    return 1;
   case FL_RELEASE:
     // Mouse must either be held down/dragged some, or this must be
     // the second click (not the one that popped up the menu):
-    if (!Fl::event_is_click() || pp.state == PUSH_STATE ||
-	pp.menubar && pp.current_item && !pp.current_item->submenu() // button
+    if (   !Fl::event_is_click() 
+        || pp.state == PUSH_STATE 
+        || (pp.menubar && pp.current_item && !pp.current_item->submenu()) // button
 	) {
 #if 0 // makes the check/radio items leave the menu up
       const Fl_Menu_Item* m = pp.current_item;
@@ -745,17 +823,30 @@ int menuwindow::early_hide_handle(int e) {
   return Fl_Window::handle(e);
 }
 
+/**
+  Pulldown() is similar to popup(), but a rectangle is
+  provided to position the menu.  The menu is made at least W
+  wide, and the picked item is centered over the rectangle
+  (like Fl_Choice uses).  If picked is zero or not
+  found, the menu is aligned just below the rectangle (like a pulldown
+  menu).
+  <P>The title and menubar arguments are used
+  internally by the Fl_Menu_Bar widget.
+*/
 const Fl_Menu_Item* Fl_Menu_Item::pulldown(
     int X, int Y, int W, int H,
     const Fl_Menu_Item* initial_item,
     const Fl_Menu_* pbutton,
     const Fl_Menu_Item* t,
-    int menubar) const
-{
+    int menubar) const {
   Fl_Group::current(0); // fix possible user error...
 
+  // track the Fl_Menu_ widget to make sure we notice if it gets
+  // deleted while the menu is open (STR #3503)
+  Fl_Widget_Tracker wp((Fl_Widget *)pbutton);
+
   button = pbutton;
-  if (pbutton) {
+  if (pbutton && pbutton->window()) {
     for (Fl_Window* w = pbutton->window(); w; w = w->window()) {
       X += w->x();
       Y += w->y();
@@ -783,32 +874,39 @@ const Fl_Menu_Item* Fl_Menu_Item::pulldown(
   if (menubar) {
     // find the initial menu
     if (!mw.handle(FL_DRAG)) {
-      Fl::release();
+      Fl::grab(0);
       return 0;
     }
   }
   initial_item = pp.current_item;
   if (initial_item) goto STARTUP;
 
-  // the main loop, runs until p.state goes to DONE_STATE:
+  // the main loop: runs until p.state goes to DONE_STATE or the menu
+  // widget is deleted (e.g. from a timer callback, see STR #3503):
   for (;;) {
 
     // make sure all the menus are shown:
-    {for (int k = menubar; k < pp.nummenus; k++)
-      if (!pp.p[k]->shown()) {
-	if (pp.p[k]->title) pp.p[k]->title->show();
-	pp.p[k]->show();
+    {
+      for (int k = menubar; k < pp.nummenus; k++) {
+        if (!pp.p[k]->shown()) {
+	  if (pp.p[k]->title) pp.p[k]->title->show();
+	  pp.p[k]->show();
+        }
       }
     }
 
     // get events:
-    {const Fl_Menu_Item* oldi = pp.current_item;
-    Fl::wait();
-    if (pp.state == DONE_STATE) break; // done.
-    if (pp.current_item == oldi) continue;}
-    // only do rest if item changes:
+    {
+      const Fl_Menu_Item* oldi = pp.current_item;
+      Fl::wait();
+      if (pbutton && wp.deleted()) // menu widget has been deleted (STR #3503)
+	break;
+      if (pp.state == DONE_STATE) break; // done.
+      if (pp.current_item == oldi) continue;
+    }
 
-    delete pp.fakemenu; pp.fakemenu = 0; // turn off "menubar button"
+    // only do rest if item changes:
+    if(pp.fakemenu) {delete pp.fakemenu; pp.fakemenu = 0;} // turn off "menubar button"
 
     if (!pp.current_item) { // pointing at nothing
       // turn off selection in deepest menu, but don't erase other menus:
@@ -816,7 +914,7 @@ const Fl_Menu_Item* Fl_Menu_Item::pulldown(
       continue;
     }
 
-    delete pp.fakemenu; pp.fakemenu = 0;
+    if(pp.fakemenu) {delete pp.fakemenu; pp.fakemenu = 0;}
     initial_item = 0; // stop the startup code
     pp.p[pp.menu_number]->autoscroll(pp.item_number);
 
@@ -854,10 +952,12 @@ const Fl_Menu_Item* Fl_Menu_Item::pulldown(
 	if (n->selected>=0) {
 	  int dy = n->y()-nY;
 	  int dx = n->x()-nX;
+	  int waX, waY, waW, waH;
+	  Fl::screen_work_area(waX, waY, waW, waH, X, Y);
 	  for (int menu = 0; menu <= pp.menu_number; menu++) {
 	    menuwindow* tt = pp.p[menu];
-	    int nx = tt->x()+dx; if (nx < 0) {nx = 0; dx = -tt->x();}
-	    int ny = tt->y()+dy; if (ny < 0) {ny = 0; dy = -tt->y();}
+	    int nx = tt->x()+dx; if (nx < waX) {nx = waX; dx = -tt->x() + waX;}
+	    int ny = tt->y()+dy; if (ny < waY) {ny = waY; dy = -tt->y() + waY;}
 	    tt->position(nx, ny);
 	  }
 	  setitem(pp.nummenus-1, n->selected);
@@ -872,7 +972,8 @@ const Fl_Menu_Item* Fl_Menu_Item::pulldown(
 	// delete all the old menus and create new one:
 	while (pp.nummenus > pp.menu_number+1) delete pp.p[--pp.nummenus];
 	pp.p[pp.nummenus++]= new menuwindow(menutable, nX, nY,
-					  title?1:0, 0, 0, title, 0, menubar, cw.x());
+					  title?1:0, 0, 0, title, 0, menubar, 
+					    (title ? 0 : cw.x()) );
       }
     } else { // !m->submenu():
       while (pp.nummenus > pp.menu_number+1) delete pp.p[--pp.nummenus];
@@ -886,35 +987,65 @@ const Fl_Menu_Item* Fl_Menu_Item::pulldown(
       }
     }
   }
-  const Fl_Menu_Item* m = pp.current_item;
-  Fl::release();
+  const Fl_Menu_Item* m = (pbutton && wp.deleted()) ? NULL : pp.current_item;
   delete pp.fakemenu;
   while (pp.nummenus>1) delete pp.p[--pp.nummenus];
   mw.hide();
+  Fl::grab(0);
   return m;
 }
 
-const Fl_Menu_Item*
-Fl_Menu_Item::popup(
+/**
+  This method is called by widgets that want to display menus.
+
+  The menu stays up until the user picks an item or dismisses it.
+  The selected item (or NULL if none) is returned. <I>This does not
+  do the callbacks or change the state of check or radio items.</I>
+
+  X,Y is the position of the mouse cursor, relative to the
+  window that got the most recent event (usually you can pass 
+  Fl::event_x() and Fl::event_y() unchanged here).
+
+  \p title is a character string title for the menu.  If
+  non-zero a small box appears above the menu with the title in it.
+
+  The menu is positioned so the cursor is centered over the item 
+  picked.  This will work even if \p picked is in a submenu.
+  If \p picked is zero or not in the menu item table the menu is
+  positioned with the cursor in the top-left corner.
+
+  \p button is a pointer to an Fl_Menu_ from which the color and
+  boxtypes for the menu are pulled.  If NULL then defaults are used.
+*/
+const Fl_Menu_Item* Fl_Menu_Item::popup(
   int X, int Y,
   const char* title,
   const Fl_Menu_Item* picked,
-  const Fl_Menu_* but
-  ) const
-{
+  const Fl_Menu_* button
+  ) const {
   static Fl_Menu_Item dummy; // static so it is all zeros
   dummy.text = title;
-  return pulldown(X, Y, 0, 0, picked, but, title ? &dummy : 0);
+  return pulldown(X, Y, 0, 0, picked, button, title ? &dummy : 0);
 }
 
-// Search only the top level menu for a shortcut.  Either &x in the
-// label or the shortcut fields are used:
-const Fl_Menu_Item* Fl_Menu_Item::find_shortcut(int* ip) const {
-  const Fl_Menu_Item* m = first();
-  if (m) for (int ii = 0; m->text; m = m->next(), ii++) {
-    if (m->activevisible()) {
+/**
+  Search only the top level menu for a shortcut.  
+  Either &x in the label or the shortcut fields are used.
+
+  This tests the current event, which must be an FL_KEYBOARD or 
+  FL_SHORTCUT, against a shortcut value.
+
+  \param ip returns the index of the item, if \p ip is not NULL.
+  \param require_alt if true: match only if Alt key is pressed.
+
+  \return found Fl_Menu_Item or NULL
+*/
+const Fl_Menu_Item* Fl_Menu_Item::find_shortcut(int* ip, const bool require_alt) const {
+  const Fl_Menu_Item* m = this;
+  if (m) for (int ii = 0; m->text; m = next_visible_or_not(m), ii++) {
+    if (m->active()) {
       if (Fl::test_shortcut(m->shortcut_)
-	 || Fl_Widget::test_shortcut(m->text)) {
+	 || Fl_Widget::test_shortcut(m->text, require_alt)) {
 	if (ip) *ip=ii;
 	return m;
       }
@@ -925,11 +1056,19 @@ const Fl_Menu_Item* Fl_Menu_Item::find_shortcut(int* ip) const {
 
 // Recursive search of all submenus for anything with this key as a
 // shortcut.  Only uses the shortcut field, ignores &x in the labels:
+/**
+  This is designed to be called by a widgets handle() method in
+  response to a FL_SHORTCUT event.  If the current event matches
+  one of the items shortcut, that item is returned.  If the keystroke
+  does not match any shortcuts then NULL is returned.  This only
+  matches the shortcut() fields, not the letters in the title
+  preceeded by '
+*/
 const Fl_Menu_Item* Fl_Menu_Item::test_shortcut() const {
-  const Fl_Menu_Item* m = first();
+  const Fl_Menu_Item* m = this;
   const Fl_Menu_Item* ret = 0;
-  if (m) for (; m->text; m = m->next()) {
-    if (m->activevisible()) {
+  if (m) for (; m->text; m = next_visible_or_not(m)) {
+    if (m->active()) {
       // return immediately any match of an item in top level menu:
       if (Fl::test_shortcut(m->shortcut_)) return m;
       // if (Fl_Widget::test_shortcut(m->text)) return m;
@@ -945,5 +1084,5 @@ const Fl_Menu_Item* Fl_Menu_Item::test_shortcut() const {
 }
 
 //
-// End of "$Id: Fl_Menu.cxx 6478 2008-10-19 21:28:06Z matt $".
+// End of "$Id$".
 //
