@@ -1,24 +1,15 @@
 //
-// "$Id: Fl_Window_hotspot.cxx 5697 2007-02-13 14:38:43Z matt $"
+// "$Id$"
 //
 // Common hotspot routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -41,7 +32,7 @@ void Fl_Window::hotspot(int X, int Y, int offscreen) {
   // stays on the screen, if possible.
   if (!offscreen) {
     int scr_x, scr_y, scr_w, scr_h;
-    Fl::screen_xywh(scr_x, scr_y, scr_w, scr_h);
+    Fl::screen_work_area(scr_x, scr_y, scr_w, scr_h);
 
     int top = 0;
     int left = 0;
@@ -73,10 +64,10 @@ void Fl_Window::hotspot(int X, int Y, int offscreen) {
 #endif
     }
     // now insure contents are on-screen (more important than border):
-    if (X+w()+right > scr_w-scr_x) X = scr_w-scr_x-right-w();
-    if (X-left < scr_x) X = left;
-    if (Y+h()+bottom > scr_h-scr_y) Y = scr_h-scr_y-bottom-h();
-    if (Y-top < scr_y) Y = top;
+    if (X+w()+right > scr_w+scr_x) X = scr_w+scr_x-right-w();
+    if (X-left < scr_x) X = left + scr_x;
+    if (Y+h()+bottom > scr_h+scr_y) Y = scr_h+scr_y-bottom-h();
+    if (Y-top < scr_y) Y = top + scr_y;
     // make sure that we will force this position
     if (X==x()) x(X-1);
   }
@@ -96,5 +87,5 @@ void Fl_Window::hotspot(const Fl_Widget *o, int offscreen) {
 
 
 //
-// End of "$Id: Fl_Window_hotspot.cxx 5697 2007-02-13 14:38:43Z matt $".
+// End of "$Id$".
 //
