@@ -1,30 +1,20 @@
 //
-// "$Id: forms.cxx 7903 2010-11-28 21:06:39Z matt $"
+// "$Id$"
 //
 // Another forms demo for the Fast Light Tool Kit (FLTK).
 //
-// This is an XForms program with very few changes.
-// Search for "fltk" to find all changes necessary to port to fltk.
+// This is an XForms program with some changes for FLTK.
 //
 // This demo show the different boxtypes. Note that some
 // boxtypes are not appropriate for some objects
 //
-// Copyright 1998-2010 by Bill Spitzak and others.
+// Copyright 1998-2015 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -36,14 +26,7 @@
 static int border = 1; // changed from FL_TRANSIENT for fltk
 // (this is so the close box and Esc work to close the window)
 
-typedef struct { int val; const char *name; } VN_struct;
-#define VN(a) {a,#a}
-
-// static VN_struct gmode[] =
-// {
-//   VN(StaticGray), VN(GrayScale), VN(StaticColor),
-//   VN(PseudoColor), VN(TrueColor), VN(DirectColor),
-// };
+typedef struct { Fl_Boxtype val; const char *name; } VN_struct;
 
 static VN_struct btypes[]=
 {
@@ -67,8 +50,9 @@ static VN_struct btypes[]=
    {FL_PLASTIC_DOWN_BOX,"plastic downbox"},
    {FL_GTK_UP_BOX,"GTK up box"},
    {FL_GTK_ROUND_UP_BOX,"GTK round up box"},
+   {FL_GLEAM_UP_BOX,"Gleam up box"},
    /* sentinel */
-   {-1}
+   {(Fl_Boxtype)(-1)}
 };
 
 #include "pixmaps/srs.xbm"
@@ -78,7 +62,7 @@ static VN_struct btypes[]=
 FL_FORM *form;
 Fl_Widget *tobj[18], *exitob, *btypeob, *modeob;
 
-void 
+void
 boxtype_cb (Fl_Widget * ob, long)
 {
   int i, req_bt = fl_get_choice(ob) - 1;
@@ -89,39 +73,21 @@ boxtype_cb (Fl_Widget * ob, long)
      fl_freeze_form (form);
      fl_redraw_form (form);
      for (i = 0; i < 18; i++)
-        fl_set_object_boxtype (tobj[i], (Fl_Boxtype)btypes[req_bt].val);
+        fl_set_object_boxtype (tobj[i], btypes[req_bt].val);
      fl_unfreeze_form (form);
      lastbt = req_bt;
      fl_redraw_form(form); // added for fltk
   }
 }
 
-void 
-mode_cb (Fl_Widget *, long)
-{
-//   static int lval = -1;
-//   int val = fl_get_choice (ob) -1;
-//   int  db = 0;
-
-//   if (val == lval || val < 0)
-//     return;
-
-//   fl_hide_form (form);
-//   if (!fl_mode_capable (gmode[val].val, 0))
-//   {
-//       fl_set_choice(ob, lval);
-//       val = lval;
-//   }
-
-//   fl_set_graphics_mode (gmode[val].val, db);
-//   fl_show_form (form, FL_PLACE_GEOMETRY, border, "Box types");
-
-//   lval = val;
+void
+mode_cb (Fl_Widget *, long) {
+  // empty
 }
 
 /*************** Creation Routines *********************/
 
-void 
+void
 create_form_form (void)
 {
   Fl_Widget *obj;
@@ -163,7 +129,7 @@ create_form_form (void)
 }
 /*---------------------------------------*/
 
-void 
+void
 create_the_forms (void)
 {
   create_form_form ();
@@ -214,18 +180,6 @@ main (int argc, char *argv[])
   for ( vn = btypes; vn->val >= 0; vn++)
     fl_addto_choice(btypeob, vn->name);
 
-//   {
-//     int i;
-//     VN_struct *g = gmode, *gs = g + sizeof (gmode) / sizeof (gmode[0]);
-//     for (i = 1; g < gs; g++, i++)
-//     {
-//       fl_addto_choice (modeob, g->name);
-//       if(!fl_mode_capable(g->val, 0))
-//         fl_set_choice_item_mode(modeob, i, FL_PUP_GRAY);
-//     }
-//   }
-//   fl_set_choice (modeob, fl_vmode+1);
-
   fl_show_form (form, FL_PLACE_MOUSE, border, "Box types");
 
   while (fl_do_forms () != exitob)
@@ -235,5 +189,5 @@ main (int argc, char *argv[])
 }
 
 //
-// End of "$Id: forms.cxx 7903 2010-11-28 21:06:39Z matt $".
+// End of "$Id$".
 //
